import 'package:flutter/material.dart';
import 'package:localization/localization.dart';
import 'package:loris/dialogues/conversations.dart';
import 'package:loris/dialogues/followreqs.dart';
import '../../global.dart' as global;

class AccountSettings extends StatelessWidget {
  const AccountSettings({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    List<Widget> children = [];
    for (int i = 0; i < global.settings!.identities.length; i++) {
      children.add(
          LogoutButton(identity: global.settings!.identities.keys.toList()[i]));
    }
    children.add(const NewAccountButton());
    children.add(const FollowreqButton());
    children.add(const ConversationButton());
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: children,
    );
  }
}

class LogoutButton extends StatelessWidget {
  const LogoutButton({
    required this.identity,
    Key? key,
  }) : super(key: key);
  final String identity;

  @override
  Widget build(BuildContext context) {
    return Wrap(
      children: [
        SelectableText(identity),
        TextButton.icon(
          onPressed: () async {
            await logout(context, identity);
          },
          icon: const Icon(Icons.logout),
          label: Text(
            "logout".i18n(),
          ),
        ),
      ],
    );
  }
}

class NewAccountButton extends StatelessWidget {
  const NewAccountButton({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return TextButton.icon(
      onPressed: (() => addNewIdentity(context)),
      icon: const Icon(Icons.person_add),
      label: Text(
        "add-account".i18n(),
      ),
    );
  }
}

void addNewIdentity(context) {
  Navigator.of(context).pushReplacementNamed("/login");
}

Future<void> logout(context, String identity) async {
  await global.settings!.removeIdentity(identity);
  if (global.settings!.identities.isEmpty) {
    Navigator.of(context).pushReplacementNamed("/login");
  } else {
    (Navigator.of(context).pushReplacementNamed("/"));
  }
}

class FollowreqButton extends StatelessWidget {
  const FollowreqButton({super.key});

  @override
  Widget build(BuildContext context) {
    return TextButton.icon(
      onPressed: (() => showDialog(
            barrierColor: Colors.transparent,
            context: context,
            builder: (context) => const FollowReqScreen(),
          )),
      icon: const Icon(Icons.check),
      label: Text(
        "follow-requests".i18n(),
      ),
    );
  }
}

class ConversationButton extends StatelessWidget {
  const ConversationButton({super.key});

  @override
  Widget build(BuildContext context) {
    return TextButton.icon(
        onPressed: () => showDialog(
              context: context,
              builder: (context) => const Chat(),
            ),
        icon: const Icon(Icons.chat),
        label: Text("chat".i18n()));
  }
}
