import 'package:flutter/material.dart';
import 'package:localization/localization.dart';
import 'package:loris/pages/notifications/notifications.dart';
import 'package:loris/pages/search/search.dart';
import 'package:loris/pages/timeline/timeline.dart';
import 'package:loris/pages/settings/settings.dart';
import '../business_logic/websocket.dart' as websocket;

class MainScaffold extends StatefulWidget {
  const MainScaffold({Key? key}) : super(key: key);

  @override
  State<MainScaffold> createState() => _MainScaffoldState();
}

class _MainScaffoldState extends State<MainScaffold> {
  int index = 0;
  int unreadNotifs = 0;

  @override
  void initState() {
    notifStream.stream.listen((event) {
      if (index != 2) {
        if (mounted) {
          setState(() {
            unreadNotifs = event;
          });
        }
      }
    });
    websocket.reloadWebsockets();
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final screens = [
      const Timeline(),
      const SearchPage(),
      const Notifications(),
      settings(context),
    ];
    return SafeArea(
      child: Scaffold(
        extendBody: false,
        body: IndexedStack(
          index: index,
          children: screens,
        ),
        floatingActionButtonLocation: FloatingActionButtonLocation.centerDocked,
        bottomNavigationBar: SizedBox(
          height: 52,
          child: BottomAppBar(
            child: NavigationBar(
                onDestinationSelected: (index) => setState(() {
                      this.index = index;
                      if (index == 2) {
                        unreadNotifs = 0;
                      }
                    }),
                selectedIndex: index,
                destinations: [
                  NavigationDestination(
                      icon: const Icon(Icons.forum), label: "timeline".i18n()),
                  NavigationDestination(
                      icon: const Icon(Icons.search), label: "search".i18n()),
                  NavigationDestination(
                      icon: Icon((unreadNotifs >= 1)
                          ? Icons.notifications_active
                          : Icons.notifications),
                      label: "notifications".i18n()),
                  NavigationDestination(
                      icon: const Icon(Icons.settings),
                      label: "settings".i18n()),
                ]),
          ),
        ),
      ),
    );
  }
}
